#include "manager.h"
#include "common.h"


///////////////////////////////////////////////////////////////////////////////


CCManager::CCManager()
{
  dprint(L_DBG, "Manager : Constructor.");
  
  receiver  = NULL;
  parser    = NULL;
  interpreter = NULL;
  
  currentDevice = NULL;
  currentChannel = NULL;
   
  osd = new CCOsd();
    
  mutex     = new cMutex;
  stopMutex = new cMutex;
  
  OsdController.Subscribe(9000,this);

  requestStop = false;
  
  Start();
}


//-----------------------------------------------------------------------------

CCManager::~CCManager()
{
  mutex->Lock();
  OsdController.Unsubscribe(this);
  Cancel(3);
  
  delete receiver;
  delete parser;
  delete interpreter;
  delete osd;
     
  mutex->Unlock();
  delete mutex;
  delete stopMutex;
  dprint(L_DBG, "Manager : Destructor.");
}


//-----------------------------------------------------------------------------

void CCManager::ChannelSwitch(const cDevice* device, int channelNumber)
{
  if (device != cDevice::ActualDevice()) return;
  
  static bool wasRunning = false;
  
  currentDevice = device;
  currentChannel = Channels.GetByNumber(channelNumber);
    
  if (channelNumber)
  {
    dprint(L_MSG, "Manager : Channel Switched to %d", channelNumber);
    if (config.StartType == S_AUTOMATIC || wasRunning) // Automatic
      StartCC();
      
    wasRunning = false;  
  }
  else
  {
    if(receiver) {
      Stop();
      wasRunning = true;
    }  
  }
}


//-----------------------------------------------------------------------------

void CCManager::SetVolume(int Volume, bool Absolute)
{
  if (config.StartType != 2) return; // Start on Mute
  
  if (Volume == 0 && Absolute) // Mute
  {
    if (!receiver) // Already started from manual start
      StartCC();
  }
  else
  {
    if(receiver) Stop();
  }
}


//-----------------------------------------------------------------------------

void CCManager::Replaying(const cControl *Control, const char *Name, const char *FileName, bool On)
{
  static bool wasRunning = false;
  
  if (On && receiver) 
  {
    dprint(L_MSG, "Replay started (%s). Suspending CC.", Name ? Name : "No Name");
    wasRunning = true;
    Stop();
  }
  else if (!On && wasRunning)
  {
    wasRunning = false;
    
    // If we started on mute, make sure mute is still active
    if (config.StartType == S_ONMUTE) 
      if (!cDevice::PrimaryDevice()->IsMute())  
        return;
        
    dprint(L_MSG, "Replay stopped (%s). Resuming CC.", Name ? Name : "No Name");
    
    if (config.StartType != S_AUTOMATIC) // SetChannel will get called
      StartCC();  
  }
}


//-----------------------------------------------------------------------------

void CCManager::StartCC()
{
  if (!currentChannel) { dprint(L_ERR, "Manager : Start() Got NULL channel. Aborting"); return; }
   
  dprint(L_DBG, "Manager : Start");
  
  int apid[2] = {currentChannel->Apid(0), 0};
  int dpid[2] = {currentChannel->Dpid(0), 0};
  
  cMutexLock(mutex);
  
  if (parser)   { dprint(L_ERR, "Manager : Start() parser is not NULL.");  return; }
  if (receiver) { dprint(L_ERR, "Manager : Start() receiver is not NULL.");return; }
  if (interpreter) { dprint(L_ERR, "Manager : Start() interpreter is not NULL.");return; } 
   
  interpreter = new Interpreter(osd); 
  parser = new CCParser(this, interpreter);  
  
#if VDRVERSNUM >= 10500      
  receiver = new CCReceiver(parser, currentChannel->GetChannelID(), currentChannel->Vpid(), apid, dpid);
#else
  receiver = new CCReceiver(parser, currentChannel->Ca(), currentChannel->Vpid(), apid, dpid);
#endif
       
  receiver->Attach( (cDevice*) currentDevice );   
  parser->Start();
  interpreter->Start();        
}


//-----------------------------------------------------------------------------

void CCManager::Stop(void)
{
  cMutexLock(mutex);
  dprint(L_DBG, "Manager : Stop");
  if (receiver) {    
    receiver->Detach();  
    delete receiver;
    receiver = NULL;
  }

  if (parser) {
    delete parser;
    parser = NULL;
  }  

  if (interpreter) {
    delete interpreter;
    interpreter = NULL;
  }
  
  osd->Reset();
}


//-----------------------------------------------------------------------------

void CCManager::NoDataFound()
{
  dprint(L_MSG, "Manager : Parser reports no CC data found. Requesting Stop.");
  cMutexLock(stopMutex);
  requestStop = true; 
}


//-----------------------------------------------------------------------------

void CCManager::Show()
{
  if(receiver) {
    dprint(L_DBG, "Manager : Showing OSD");
    osd->Show();
  }  
}


//-----------------------------------------------------------------------------

void CCManager::Hide()
{
  dprint(L_DBG, "Manager : Hiding OSD");
  osd->Hide();
}
  
  
//-----------------------------------------------------------------------------
  
void CCManager::SetOsdColors(tColor bg, tColor fg)
{
  osd->SetBgColor(bg);
  osd->SetFgColor(fg);
}


//-----------------------------------------------------------------------------

void CCManager::Action(void)
{
  dprint(L_DBG, "Manager : Monitor Thread Started.");
  while (Running())
  {
    cCondWait::SleepMs(500);
    cMutexLock(stopMutex);
    if (requestStop) {
      requestStop = false;
      dprint(L_DBG, "Manager : Stopping Data Aquisition.");
      CCManager::Stop();
    }
  }
  dprint(L_DBG, "Manager : Monitor Thread Ended.");
}


///////////////////////////////////////////////////////////////////////////////

